#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <vector>
#include <string>
#include <array>
#include <future>
#include <iostream>
#include <cinttypes>


#define RUN_CHECK_FAILED 6

static const char *program_name;

using namespace std;

/*
  arguments: user-program
 */
int main(int argc, char *argv[])
{
    program_name = argv[0];
    signal(SIGPIPE, SIG_DFL);

    vector<string> input_dict;
    vector<string> query_dict;
    array<uint64_t, 1024> data;

    int n;
    cin >> n;
    input_dict.reserve(n);
    for (int i = 0; i < n; ++i) {
        string s;
        cin >> s;
        input_dict.push_back(s);
    }
    cin >> n;
    query_dict.reserve(n);
    for (int i = 0; i < n; ++i) {
        string s;
        cin >> s;
        query_dict.push_back(s);
    }
 

    /*
    fprintf(stderr, "runpipe command line args:\n");
    for (int i = 0; i < argc; ++i) {
        fprintf(stderr, "[%d]: '%s'\n", i, argv[i]);
    }
    */
    bool special_mode = false;
    int noise_offset = 2;
    if (strstr(argv[1], "runmono") || strstr(argv[1], "runjava")) {
        special_mode = true;
        noise_offset = 3;
    }

    int input_pipe[2];
    int pfd1[2];
    if (pipe(pfd1) < 0 || pipe(input_pipe)) {
        fprintf(stderr, "%s: pipe failed: %s\n", program_name, strerror(errno));
        exit(RUN_CHECK_FAILED);
    }
    int pid1 = fork();
    if (pid1 < 0) {
        fprintf(stderr, "%s: fork failed: %s\n", program_name, strerror(errno));
        exit(RUN_CHECK_FAILED);
    }
    if (!pid1) {
        signal(SIGPIPE, SIG_IGN);
        dup2(pfd1[1], 1);
        dup2(input_pipe[0], 0);
        close(pfd1[1]); close(pfd1[0]);
        close(input_pipe[0]); close(input_pipe[1]);
        if (special_mode) {
            execlp(argv[1], argv[1], argv[2], NULL);
        } else {
            execlp(argv[1], argv[1], NULL);
        }
        fprintf(stderr, "%s: execlp failed: %s\n", program_name, strerror(errno));
        exit(RUN_CHECK_FAILED);
    }
    close(pfd1[1]);

    int pfd2[2];
    if (pipe(pfd2) < 0) {
        fprintf(stderr, "%s: pipe failed: %s\n", program_name, strerror(errno));
        close(pfd1[0]);
        kill(pid1, SIGKILL);
        wait(NULL);
        exit(RUN_CHECK_FAILED);
    }

    int pid3 = fork();
    if (pid3 < 0) {
        close(pfd2[0]);
        kill(pid1, SIGKILL);
        wait(NULL);
        wait(NULL);
        exit(RUN_CHECK_FAILED);
    }
    if (!pid3) {
        signal(SIGPIPE, SIG_IGN);
        dup2(pfd2[0], 0); close(pfd2[0]);
        if (special_mode) {
            execlp(argv[1], argv[1], argv[2], NULL);
        } else {
            execlp(argv[1], argv[1], NULL);
        }
        fprintf(stderr, "%s: execlp failed: %s\n", program_name, strerror(errno));
        exit(RUN_CHECK_FAILED);
    }

    auto f_read = std::async(std::launch::async, [&] () -> void {
        FILE *f_input_pipe = fdopen(input_pipe[1], "w");
        fprintf(f_input_pipe, "1\n%d\n", input_dict.size());
        for (auto &s : input_dict) {
            fprintf(f_input_pipe, "%s\n", s.data());
        }
        fclose(f_input_pipe);
    });

    auto f_transfer = std::async(std::launch::async, [&] () -> void {
        FILE *pipe_in = fdopen(pfd1[0], "r");
        FILE *pipe_out = fdopen(pfd2[1], "w");
        fprintf(pipe_out, "2\n");
        cerr << "storage begin\n";
        for (int i = 0; i < 1024; ++i) {
            uint64_t val;
            int cnt = fscanf(pipe_in, "%" SCNu64, &val);

            if (cnt != 1) {
                kill(pid1, SIGKILL);
                kill(pid3, SIGKILL);        
                exit(2);
            }

            fprintf(pipe_out, "%" PRIu64, val);
            cerr << val << " ";
            if (i != 1023) {
                fputc(' ', pipe_out);
            } else {
                fputc('\n', pipe_out);
            }
        }
        cerr << "\nstorage read end" << endl;
        fclose(pipe_in);

        fprintf(pipe_out, "%d\n", query_dict.size());

        for (auto &s : query_dict) {
            fprintf(pipe_out, "%s\n", s.data());
        }
        cerr << "storage write end" << endl;
        fclose(pipe_out);
    });

    int status1 = 0, status2 = 0, status3 = 0;
    waitpid(pid1, &status1, 0);
    cerr << "proc1 finished" << endl;
    waitpid(pid3, &status3, 0);
    char c;
    cerr << "proc2 finished" << endl;
    while (read(pfd2[0], &c, sizeof(c)) > 0) {}
    while (read(input_pipe[0], &c, sizeof(c)) > 0) {}     
    cerr << "read finished" << endl;
    f_read.get();
    cerr << "read thread finished" << endl;
    f_transfer.get();
    cerr << "transfer thread finished" << endl;
 
    if (WIFSIGNALED(status2)) {
        fprintf(stderr, "%s: noise program terminated with signal %d\n", program_name, WTERMSIG(status2));
        exit(RUN_CHECK_FAILED);
    }

    if (WIFSIGNALED(status1)) {
        fprintf(stderr, "%s: encoder terminated with signal %d\n", program_name, WTERMSIG(status1));
        exit(1);
    }
    if (WEXITSTATUS(status1)) {
        fprintf(stderr, "%s: encoder exited with code %d\n", program_name, WEXITSTATUS(status1));
        exit(1);
    }

    if (WEXITSTATUS(status2)) {
        fprintf(stderr, "%s: noise program exited with code %d\n", program_name, WEXITSTATUS(status2));
        exit(WEXITSTATUS(status2));
    }

    if (WIFSIGNALED(status3)) {
        fprintf(stderr, "%s: decoder terminated with signal %d\n", program_name, WTERMSIG(status3));
        exit(1);
    }
    if (WEXITSTATUS(status3)) {
        fprintf(stderr, "%s: decoder exited with code %d\n", program_name, WEXITSTATUS(status3));
        exit(1);
    }

    return 0;
}
