#include <vector>
#include <iostream>
#include <string>
#include <functional>

using namespace std;

unsigned hash1(const string &str)
{
    unsigned hash = 5381;
    int c;

    for (auto c : str) {
        hash = ((hash << 5) + hash) + c; /* hash * 33 + c */
    }
    return hash;
}

unsigned jenkins(const string &str)
{
    unsigned hash, i;
    for(hash = i = 0; i < str.size(); ++i)
    {
        hash += str[i];
        hash += (hash << 10);
        hash ^= (hash >> 6);
    }
    hash += (hash << 3);
    hash ^= (hash >> 11);
    hash += (hash << 15);
    return hash;
}

unsigned murmur(const string &str)
{
  unsigned h(3323198485ul);
  for (auto c: str) {
    h ^= c;
    h *= 0x5bd1e995;
    h ^= h >> 15;
  }
  return h;
}

unsigned hash2(const string &str) {
    const int p = 31;
    const int m = 1e9 + 9;
    long long hash_value = 0;
    long long p_pow = 1;
    for (auto c : str) {
        hash_value = (hash_value + (c - 'a' + 1) * p_pow) % m;
        p_pow = (p_pow * p) % m;
    }
    return hash_value;
}

unsigned hash3(const string &str) {
    return std::hash<string>()(str);
}

struct blumm {

    vector<function<unsigned(const string&)>> functions = {hash1, jenkins, murmur, hash2};

    vector<uint64_t> bits;

    blumm() : bits(1024) {}

    static constexpr int size = 1024 * 64;

    void add(const string& s) {
        for (const auto &f : functions) {
            int bit = f(s) % size;
            bits[bit / 64] |= 1 << (bit % 64);
        }
    }

    bool test(const string& s) {
        for (const auto &f : functions) {
            int bit = f(s) % size;
            if ((bits[bit / 64] & 1 << (bit % 64)) == 0 ) {
                return false;
            }
        }
        return true;
    }
};

int main() {
    int mode;
    cin >> mode;

    if (mode == 1) {
        blumm b;
        int n;
        cin >> n;
        for (int i = 0; i < n; ++i) {
            string s;
            cin >> s;
            b.add(s);
        }
        for (auto i : b.bits) {
            cout << i << " ";
        }
    } else {
        blumm b;
        for (int i = 0; i < 1024; ++i) {
            cin >> b.bits[i];
        }
        int k;
        cin >> k;
        for (int i = 0; i < k; ++i) {
            string s;
            cin >> s;
            if (!b.test(s)) {
                cout << "NO\n";
            } else {
                cout << "YES\n";
            }
        }
    }
}
