/* -*- mode: c++; c-basic-offset: 4 -*- */

#include <iostream>
#include <string>
#include <sstream>
#include <random>
#include <vector>
#include <cstdint>
#include <cstdlib>

using namespace std;

random_device rnd_dev;
mt19937 random_gen;

class SimpleMap
{
public:
    struct Entry
    {
        uint64_t key;
        int index;
        int value;
    };

private:
    int size_ = 0;
    Entry entries_[3];

public:
    void inc(uint64_t key, int index)
    {
        if (size_ == 3) {
            if (entries_[2].key == key) {
                ++entries_[2].value;
                return;
            }
            if (entries_[1].key == key) {
                ++entries_[1].value;
                return;
            }
            if (entries_[0].key == key) {
                ++entries_[0].value;
                return;
            }
        } else if (size_ == 2) {
            if (entries_[1].key == key) {
                ++entries_[1].value;
                return;
            }
            if (entries_[0].key == key) {
                ++entries_[0].value;
                return;
            }
        } else if (size_ == 1) {
            if (entries_[0].key == key) {
                ++entries_[0].value;
                return;
            }
        }
        if (size_ == 3) abort();
        entries_[size_].key = key;
        entries_[size_].value = 1;
        entries_[size_].index = index;
        ++size_;
    }
    int size() const { return size_; }
    const Entry &operator [] (int i) const { return entries_[i]; }
    const Entry &get(uint64_t key) const
    {
        if (size_ == 3) {
            if (entries_[2].key == key) {
                return entries_[2];
            }
            if (entries_[1].key == key) {
                return entries_[1];
            }
            if (entries_[0].key == key) {
                return entries_[0];
            }
        } else if (size_ == 2) {
            if (entries_[1].key == key) {
                return entries_[1];
            }
            if (entries_[0].key == key) {
                return entries_[0];
            }
        } else if (size_ == 1) {
            if (entries_[0].key == key) {
                return entries_[0];
            }
        }
        abort();
    }
};

class SimpleSet
{
    int size_ = 0;
    int entries_[2];

public:
    void insert(int value)
    {
        if (size_ >= 1 && entries_[0] == value) {
            return;
        }
        if (size_ >= 2 && entries_[1] == value) {
            return;
        }
        if (size_ == 2) abort();
        entries_[size_] = value;
        ++size_;
    }
    bool contains(int value)
    {
        if (size_ >= 1 && entries_[0] == value) {
            return true;
        }
        if (size_ >= 2 && entries_[1] == value) {
            return true;
        }
        return false;
    }
    int size() const { return size_; }
    void remove(int value)
    {
        if (size_ == 2) {
            if (entries_[0] == value) {
                entries_[0] = entries_[1];
                --size_;
            } else if (entries_[1] == value) {
                --size_;
            }
        } else if (size_ == 1) {
            if (entries_[0] == value) {
                --size_;
            }
        } else {
        }
    }
    int operator[](int index) const { return entries_[index]; }
};

struct Solver
{
    constexpr static uint64_t MODULO = 1152921504606846883ULL;
    constexpr static uint64_t MULT = 5;

    int m_, l_, n_;
    int cur_ = -1;
    vector<uint64_t> hashes_;
    uint32_t init_val_{};
    SimpleSet ones_;
    vector<unsigned char> goods_;
    int answer_ = -1;

    void init_hashes()
    {
        for (auto &h : hashes_) {
            h = init_val_ % MODULO;
        }
    }

    Solver(int m, int l, int n) : m_(m), l_(l), n_(n), hashes_(n), goods_(n)
    {
        random_gen.seed(rnd_dev());
        init_val_ = random_gen();
        init_hashes();
    }

    int fromchar(char c)
    {
        if (c == 'A') {
            return 0;
        } else if (c == 'G') {
            return 1;
        } else if (c == 'U') {
            return 2;
        } else if (c == 'C') {
            return 3;
        } else {
            abort();
        }
    }

    bool process(const string &s)
    {
        if (answer_ >= 0) return true;

        ++cur_;
        for (int i = 0; i < n_; ++i) {
            uint64_t v = hashes_[i];
            v = (v * MULT) % MODULO;
            v = (v + fromchar(s[i])) % MODULO;
            hashes_[i] = v;
        }
        if (cur_ % l_ == l_ - 1) {
            /*
            cerr << "cur: " << cur_ << endl;
            for (int i = 0; i < n_; ++i) {
                cerr << "    " << hashes_[i] << endl;
            }
            */
            SimpleMap sm;
            for (int i = 0; i < n_; ++i) {
                sm.inc(hashes_[i], i);
            }
            /*
            for (int i = 0; i < sm.size(); ++i) {
                const auto &e = sm[i];
                cerr << "    " << e.key << " " << e.value << " " << e.index << endl;
            }
            */
            if (sm.size() == 3) {
                int index1 = -1;
                int index2 = -1;
                if (sm[0].value == 1 && sm[1].value > 1 && sm[2].value > 1) {
                    answer_ = sm[0].index + 1;
                    return true;
                } else if (sm[0].value > 1 && sm[1].value == 1 && sm[2].value > 1) {
                    answer_ = sm[1].index + 1;
                    return true;
                } else if (sm[0].value > 1 && sm[1].value > 1 && sm[2].value == 1) {
                    answer_ = sm[2].index + 1;
                    return true;
                } else if (sm[0].value == 1 && sm[1].value == 1 && sm[2].value > 1) {
                    index1 = sm[0].index;
                    index2 = sm[1].index;
                } else if (sm[0].value > 1 && sm[1].value == 1 && sm[2].value == 1) {
                    index1 = sm[1].index;
                    index2 = sm[2].index;
                } else if (sm[0].value == 1 && sm[1].value > 1 && sm[2].value == 1) {
                    index1 = sm[0].index;
                    index2 = sm[2].index;
                } else {
                    abort();
                }
                if (ones_.size() == 0) {
                    ones_.insert(index1);
                    ones_.insert(index2);
                } else if (ones_.size() == 2) {
                    if (ones_.contains(index1)) {
                        if (ones_.contains(index2)) {
                            // nothing yet
                        } else {
                            answer_ = index1 + 1;
                            return true;
                        }
                    } else if (ones_.contains(index2)) {
                        answer_ = index2 + 1;
                        return true;
                    } else {
                        cerr << "ambiguity" << endl;
                        exit(1);
                    }
                } else {
                    cerr << "ambiguity" << endl;
                    exit(1);
                }
            }

            init_hashes();
        }
        return false;
    }
    void print()
    {
        if (answer_ >= 0) {
            cout << answer_ << endl;
        } else {
            if (ones_.size() == 2) {
                cerr << "ambiguity" << endl;
                exit(1);
            } else {
                cout << 0 << endl;
            }
        }
    }
};

int main()
{
    int m, l, n;

    ios_base::sync_with_stdio(0);

    {
        string s;
        getline(cin, s);
        if (!cin) {
            cerr << "line 1 read error" << endl;
            return 1;
        }
        istringstream iss(s);
        if (!(iss >> m >> l >> n)) {
            cerr << "invalid line 1" << endl;
            return 1;
        }
    }
    if (m <= 0 || m > 100000) {
        cerr << "m out of range" << endl;
        return 1;
    }
    if (l < 1 || m % l != 0) {
        cerr << "invalid l" << endl;
        return 1;
    }
    if (n <= 4 || n > 100) {
        cerr << "invalid n" << endl;
        return 1;
    }

    Solver solver(m, l, n);
    for (int c = 0; c < m; ++c) {
        string s;
        getline(cin, s);
        if (!cin) {
            cerr << "read error in line " << (c + 1) << endl;
            return 1;
        }
        if (s.length() > 0 && s[s.length() - 1] == '\n') s.resize(s.length() - 1);
        if (int(s.length()) != n) {
            cerr << "invalid line " << (c + 1) << endl;
            return 1;
        }
        for (char t : s) {
            if (t != 'A' && t != 'G' && t != 'U' && t != 'C') {
                cerr << "invalid line " << (c + 1) << endl;
                return 1;
            }
        }
        solver.process(s);
    }
    {
        string s;
        getline(cin, s);
        if (!cin.eof()) {
            cerr << "garbage" << endl;
            return 1;
        }
    }
    solver.print();
    return 0;
}
